@@ -13,20 +13,42 @@
 #include <gtk/gtksignal.h>
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-event-source.h>
+#include <bonobo/bonobo-running-context.h>
+#include <time.h>
 
 static BonoboObjectClass    *bonobo_event_source_parent_class;
 POA_Bonobo_EventSource__vepv bonobo_event_source_vepv;
 
 
 struct _BonoboEventSourcePrivate {
-	GSList *listeners;  /* CONTAINS: ListenerDesc* */
+	GSList  *listeners;  /* CONTAINS: ListenerDesc* */
+	gboolean ignore;
+	gint     counter;    /* to create unique listener Ids */
 };
 
 typedef struct {
 	Bonobo_Listener listener;
+	Bonobo_EventSource_ListenerId id;
 	CORBA_char *event_mask; /* send all events if NULL */
 } ListenerDesc;
 
+/*
+ * tries to make a unique connection Id. Adding the time make an
+ * accidental remove of a listener more unlikely.
+ */
+static Bonobo_EventSource_ListenerId
+create_listener_id (BonoboEventSource *source)
+{
+	guint32 id;
+
+	source->priv->counter = source->priv->counter++ & 0x0000ffff;
+
+	if (!source->priv->counter) source->priv->counter++;
+
+	id = source->priv->counter | (time(NULL) << 16);
+
+	return id;
+}
 
 static inline BonoboEventSource * 
 bonobo_event_source_from_servant (PortableServer_Servant servant)
@@ -44,69 +66,58 @@ desc_free (ListenerDesc *desc, CORBA_Environment *ev)
 	}
 }
 
-static void
+static Bonobo_EventSource_ListenerId
 impl_Bonobo_EventSource_addListenerWithMask (PortableServer_Servant servant,
 					     const Bonobo_Listener  l,
 					     const CORBA_char      *event_mask,
 					     CORBA_Environment     *ev)
 {
 	BonoboEventSource *event_source;
-	GSList            *list;
 	CORBA_char        *mask_copy = NULL;
 	ListenerDesc      *desc;
 
-	g_return_if_fail (!CORBA_Object_is_nil (l, ev));
+	g_return_val_if_fail (!CORBA_Object_is_nil (l, ev), 0);
 
 	event_source = bonobo_event_source_from_servant (servant);
 
 	if (event_mask)
 		mask_copy = CORBA_string_dup (event_mask);
 
-	/* 
-	 * Check if listener is already registered, and if so modify 
-	 * event_mask to recieve masked events.
-	 */
-	for (list = event_source->priv->listeners; list; list = list->next) {
-		desc = (ListenerDesc *) list->data;
-
-		if (CORBA_Object_is_equivalent (l, desc->listener, ev)) {
-			CORBA_free (desc->event_mask);
-			desc->event_mask = mask_copy;
-			return;
-		}
-	}
+	if (event_source->priv->ignore) /* Hook for running context */
+		bonobo_running_context_ignore_object (l);
 
 	desc = g_new0 (ListenerDesc, 1);
 	desc->listener = bonobo_object_dup_ref (l, ev);
+	desc->id = create_listener_id (event_source);
 	desc->event_mask = mask_copy;
 
 	event_source->priv->listeners = g_slist_prepend (event_source->priv->listeners, desc);
+
+	return desc->id;
 }
 
-static void
+static Bonobo_EventSource_ListenerId
 impl_Bonobo_EventSource_addListener (PortableServer_Servant servant,
 				     const Bonobo_Listener  l,
 				     CORBA_Environment     *ev)
 {
-	impl_Bonobo_EventSource_addListenerWithMask (servant, l, NULL, ev);
+	return impl_Bonobo_EventSource_addListenerWithMask (servant, l, NULL, ev);
 }
 
 static void
 impl_Bonobo_EventSource_removeListener (PortableServer_Servant servant,
-					const Bonobo_Listener  l,
+					const Bonobo_EventSource_ListenerId id,
 					CORBA_Environment     *ev)
 {
 	BonoboEventSource *event_source;
 	GSList *list;
 
-	g_return_if_fail (!CORBA_Object_is_nil (l, ev));
-
 	event_source = bonobo_event_source_from_servant (servant);
 
 	for (list = event_source->priv->listeners; list; list = list->next) {
 		ListenerDesc *desc = (ListenerDesc *) list->data;
 
-		if (CORBA_Object_is_equivalent (l, desc->listener, ev)) {
+		if (desc->id == id) {
 			event_source->priv->listeners = 
 				g_slist_remove (event_source->priv->listeners,
 						desc);
@@ -138,7 +149,7 @@ impl_Bonobo_EventSource_removeListener (PortableServer_Servant servant,
 void
 bonobo_event_source_notify_listeners (BonoboEventSource *event_source,
 				      const char        *event_name,
-				      CORBA_any         *value,
+				      const CORBA_any   *value,
 				      CORBA_Environment *opt_ev)
 {
 	GSList *list;
@@ -163,6 +174,24 @@ bonobo_event_source_notify_listeners (BonoboEventSource *event_source,
 		CORBA_exception_free (&ev);
 }
 
+void
+bonobo_event_source_notify_listeners_full (BonoboEventSource *event_source,
+					   const char        *path,
+					   const char        *type,
+					   const char        *subtype,
+					   const CORBA_any   *value,                          
+					   CORBA_Environment *opt_ev)
+{
+	char *event_name;
+
+	event_name = bonobo_event_make_name (path, type, subtype);
+
+	bonobo_event_source_notify_listeners (event_source, event_name,
+					      value, opt_ev);
+
+	g_free (event_name);
+}
+
 
 /**
  * bonobo_event_source_get_epv:
@@ -236,6 +265,13 @@ bonobo_event_source_class_init (BonoboEventSourceClass *klass)
 	init_event_source_corba_class ();
 }
 
+/**
+ * bonobo_event_source_get_type:
+ * 
+ * Registers the GtkType for this BonoboObject.
+ * 
+ * Return value: the type.
+ **/
 GtkType
 bonobo_event_source_get_type (void)
 {
@@ -259,6 +295,14 @@ bonobo_event_source_get_type (void)
         return type;
 }
 
+/**
+ * bonobo_event_source_corba_object_create:
+ * @object: the object to tie the CORBA object to
+ * 
+ * creates the CORBA object associated with an event source
+ * 
+ * Return value: the CORBA handle to this object.
+ **/
 Bonobo_EventSource
 bonobo_event_source_corba_object_create (BonoboObject *object)
 {
@@ -282,6 +326,15 @@ bonobo_event_source_corba_object_create (BonoboObject *object)
         return bonobo_object_activate_servant (object, servant);
 }
 
+/**
+ * bonobo_event_source_construct:
+ * @event_source: 
+ * @corba_event_source: 
+ * 
+ * constructs an event source.
+ * 
+ * Return value: the constructed event source or NULL on error.
+ **/
 BonoboEventSource *
 bonobo_event_source_construct (BonoboEventSource  *event_source, 
 			       Bonobo_EventSource corba_event_source) 
@@ -327,3 +380,18 @@ bonobo_event_source_new (void)
 	return bonobo_event_source_construct (
 		event_source, corba_event_source);
 }
+
+/**
+ * bonobo_event_source_ignore_listeners:
+ * @event_source: 
+ * 
+ *  Instructs the event source to de-register any listeners
+ * that are added from the global running context.
+ **/
+void
+bonobo_event_source_ignore_listeners (BonoboEventSource *event_source)
+{
+	g_return_if_fail (BONOBO_IS_EVENT_SOURCE (event_source));
+
+	event_source->priv->ignore = TRUE;
+}
