@@ -7,7 +7,6 @@
  *	Michael Meeks    (michael@helixcode.com)
  *	Ettore Perazzoli (ettore@helixcode.com)
  */
-
 #include "bonobo.h"
 #include <liboaf/liboaf.h>
 #include <liboaf/oaf-async.h>
@@ -22,7 +21,7 @@ struct {
 	{ "OAFIID:", "OAFIID:Bonobo_Moniker_Oaf"   },
 	{ "OAFAID:", "OAFIID:Bonobo_Moniker_Oaf"   },
 	{ "new:",    "OAFIID:Bonobo_Moniker_New"   },
-	{ "http:",   "OAFIID:Bonobo_Moniker_HTTP"  },
+	{ "http:",   "OAFIID:Bonobo_Moniker_http"  },
 /*
 	{ "queue:", "" },
 */
@@ -46,27 +45,86 @@ moniker_id_from_nickname (const CORBA_char *name)
 	return NULL;
 }
 
-static char *
+/*
+ * get_full_interface_name:
+ * @ifname: original name: can be in form Bonobo/Control
+ *
+ * Return value: full name eg. IDL:Bonobo/Control:1.0
+ */
+static gchar *
+get_full_interface_name (const char *ifname)
+{
+	int len, had_ver;
+	const char *a;
+	char *retval, *b;
+
+	g_return_val_if_fail (ifname != NULL, NULL);
+
+	len = strlen (ifname);
+	retval = g_new (char, len + 4 + 4 + 1);
+
+	strcpy (retval, "IDL:");
+	a = ifname;
+	b = retval + 4;
+
+	if (ifname [0] == 'I' &&
+	    ifname [1] == 'D' &&
+	    ifname [2] == 'L' &&
+	    ifname [3] == ':')
+		a += 4;
+
+	for (had_ver = 0; (*b = *a); a++, b++) {
+		if (*a == ':')
+			had_ver = 1;
+	}
+
+	if (!had_ver)
+		strcpy (b, ":1.0");
+
+	return retval;
+}
+
+static gchar *
 query_from_name (const char *name)
 {
 	char *prefix, *query;
 	int   len;
 
 	for (len = 0; name [len]; len++) {
-		if (name [len] == ':')
+		if (name [len] == ':') {
+			len++;
 			break;
+		}
 	}
 
+
 	prefix = g_strndup (name, len);
 		
 	query = g_strdup_printf (
 		"repo_ids.has ('IDL:Bonobo/Moniker:1.0') AND "
-		"bonobo:moniker == '%s'", prefix);
+		"bonobo:moniker.has ('%s')", prefix);
 	g_free (prefix);
 
 	return query;
 }
 
+/**
+ * bonobo_moniker_util_new_from_name_full:
+ * @parent: A parent moniker to chain to or CORBA_OBJECT_NIL
+ * @name: the display name
+ * @ev: corba environment
+ * 
+ *  This routine is used to continue building up the chain
+ * that forms a multi-part moniker. The parent is referenced
+ * as the parent and passed onto the next stage of parsing
+ * the 'name'. We eventualy return a moniker handle which
+ * represents the end of a linked list of monikers each
+ * pointing to their parent:
+ *
+ * file:/tmp/a.tar.gz <-- gzip: <-- tar: <-- [ this is returned ]
+ * 
+ * Return value: The end node of a list of monikers representing @name
+ **/
 Bonobo_Moniker
 bonobo_moniker_util_new_from_name_full (Bonobo_Moniker     parent,
 					const CORBA_char  *name,
@@ -94,7 +152,7 @@ bonobo_moniker_util_new_from_name_full (Bonobo_Moniker     parent,
 
 		g_free (query);
 		
-		if (BONOBO_EX(ev))
+		if (BONOBO_EX (ev))
 			return CORBA_OBJECT_NIL;
 
 		if (object == CORBA_OBJECT_NIL) {
@@ -105,7 +163,7 @@ bonobo_moniker_util_new_from_name_full (Bonobo_Moniker     parent,
 	} else {
 		object = oaf_activate_from_id ((gchar *) iid, 0, NULL, ev);
 
-		if (BONOBO_EX(ev))
+		if (BONOBO_EX (ev))
 			return CORBA_OBJECT_NIL;
 		
 		if (object == CORBA_OBJECT_NIL) {
@@ -117,7 +175,7 @@ bonobo_moniker_util_new_from_name_full (Bonobo_Moniker     parent,
 	toplevel = Bonobo_Unknown_queryInterface (
 		object, "IDL:Bonobo/Moniker:1.0", ev);
 
-	if (BONOBO_EX(ev))
+	if (BONOBO_EX (ev))
 		return CORBA_OBJECT_NIL;
 
 	if (object == CORBA_OBJECT_NIL) {
@@ -128,16 +186,26 @@ bonobo_moniker_util_new_from_name_full (Bonobo_Moniker     parent,
 
 	moniker = Bonobo_Moniker_parseDisplayName (toplevel, parent,
 						   name, ev);
-	if (BONOBO_EX(ev))
+	if (BONOBO_EX (ev))
 		return CORBA_OBJECT_NIL;
 
 	bonobo_object_release_unref (toplevel, ev);
-	if (BONOBO_EX(ev))
+	if (BONOBO_EX (ev))
 		return CORBA_OBJECT_NIL;
 
 	return moniker;
 }
 
+/**
+ * bonobo_moniker_util_get_parent_name:
+ * @moniker: the moniker
+ * @ev: a corba exception environment
+ * 
+ *  This gets the display name of the parent moniker ( recursively
+ * all of the parents of this moniker ).
+ * 
+ * Return value: the display name; use CORBA_free to release it.
+ **/
 CORBA_char *
 bonobo_moniker_util_get_parent_name (Bonobo_Moniker     moniker,
 				     CORBA_Environment *ev)
@@ -150,13 +218,13 @@ bonobo_moniker_util_get_parent_name (Bonobo_Moniker     moniker,
 
 	parent = Bonobo_Moniker__get_parent (moniker, ev);
 
-	if (BONOBO_EX(ev) ||
+	if (BONOBO_EX (ev) ||
 	    parent == CORBA_OBJECT_NIL)
 		return NULL;
 	
 	name = Bonobo_Moniker_getDisplayName (parent, ev);
 
-	if (BONOBO_EX(ev))
+	if (BONOBO_EX (ev))
 		name = NULL;
 
 	bonobo_object_release_unref (parent, ev);
@@ -164,6 +232,18 @@ bonobo_moniker_util_get_parent_name (Bonobo_Moniker     moniker,
 	return name;
 }
 
+/**
+ * bonobo_moniker_util_qi_return:
+ * @object: the unknown to query
+ * @requested_interface: the desired interface
+ * @ev: a corba exception environment 
+ * 
+ *  A helper function to share code from the end of a resolve
+ * implementation; this ensures that the returned object is of
+ * the correct interface by doing a queryInterface on the object.
+ * 
+ * Return value: an handle to the requested interface
+ **/
 Bonobo_Unknown
 bonobo_moniker_util_qi_return (Bonobo_Unknown     object,
 			       const CORBA_char  *requested_interface,
@@ -171,7 +251,7 @@ bonobo_moniker_util_qi_return (Bonobo_Unknown     object,
 {
 	Bonobo_Unknown retval = CORBA_OBJECT_NIL;
 
-	if (BONOBO_EX(ev))
+	if (BONOBO_EX (ev))
 		return CORBA_OBJECT_NIL;
 	
 	if (object == CORBA_OBJECT_NIL) {
@@ -184,7 +264,7 @@ bonobo_moniker_util_qi_return (Bonobo_Unknown     object,
 	retval = Bonobo_Unknown_queryInterface (
 		object, requested_interface, ev);
 
-	if (BONOBO_EX(ev))
+	if (BONOBO_EX (ev))
 		goto release_unref_object;
 	
 	if (retval == CORBA_OBJECT_NIL) {
@@ -203,6 +283,20 @@ bonobo_moniker_util_qi_return (Bonobo_Unknown     object,
 		return CORBA_OBJECT_NIL;
 }
 
+/**
+ * bonobo_moniker_util_seek_std_separator:
+ * @str: the string to scan
+ * @min_idx: the minimum offset at which a separator can be found.
+ * 
+ *  This looks for a standard separator in a moniker's
+ * display name string. Most monikers will want to use
+ * standard separators.
+ *
+ *  See also bonobo_moniker_util_escape
+ * 
+ * Return value: the position of the standard separator, or a
+ * pointer to the end of the string.
+ **/
 int
 bonobo_moniker_util_seek_std_separator (const CORBA_char *str,
 					int               min_idx)
@@ -220,18 +314,126 @@ bonobo_moniker_util_seek_std_separator (const CORBA_char *str,
 		}
 	}
 
-#warning We need escaping support here 
-
 	for (; str [i]; i++) {
 
-		if (str [i] == '!' ||
-		    str [i] == '#')
+		if (str [i] == '\\' && str [i + 1])
+			i++;
+		else if (str [i] == '!' ||
+			 str [i] == '#')
 			break;
 	}
 	
 	return i;
 }
 
+/**
+ * bonobo_moniker_util_escape:
+ * @string: an unescaped string
+ * @offset: an offset of characters to ignore
+ * 
+ *  Escapes possible separator characters inside a moniker
+ * these include '!' and '#', the '\' escaping character is
+ * used.
+ * 
+ * Return value: an escaped sub-string.
+ **/
+char *
+bonobo_moniker_util_escape (const char *string, int offset)
+{
+	gchar *escaped, *p;
+	guint  backslashes = 0;
+	int    i, len;
+
+	g_return_val_if_fail (string != NULL, NULL);
+
+	len = strlen (string);
+	g_return_val_if_fail (offset < len, NULL);
+
+	for (i = offset; i < len; i++) {
+		if (string [i] == '\0')
+			break;
+		else if (string [i] == '\\' ||
+			 string [i] == '#'  ||
+			 string [i] == '!')
+			backslashes ++;
+	}
+	
+	if (!backslashes)
+		return g_strdup (&string [offset]);
+
+	p = escaped = g_new (gchar, len - offset + backslashes + 1);
+
+	for (i = offset; i < len; i++) {
+		if (string [i] == '\\' ||
+		    string [i] == '#'  ||
+		    string [i] == '!')
+			*p++ = '\\';
+		*p++ = string [i];
+	}
+	*p = '\0';
+
+	return escaped;
+}
+
+/**
+ * bonobo_moniker_util_unescape:
+ * @string: a string
+ * @num_chars: the number of chars to process.
+ * 
+ *  This routine strips @num_chars: from the start of
+ * @string, discards the rest, and proceeds to un-escape
+ * characters escaped with '\'.
+ * 
+ * Return value: the unescaped sub string.
+ **/
+char *
+bonobo_moniker_util_unescape (const char *string, int num_chars)
+{
+	gchar *escaped, *p;
+	guint  backslashes = 0;
+	int    i;
+
+	g_return_val_if_fail (string != NULL, NULL);
+
+	for (i = 0; i < num_chars; i++) {
+		if (string [i] == '\0')
+			break;
+		else if (string [i] == '\\')
+			backslashes ++;
+	}
+
+	if (!backslashes)
+		return g_strndup (string, num_chars);
+
+	p = escaped = g_new (gchar, strlen (string) - backslashes + 1);
+
+	for (i = 0; i < num_chars; i++) {
+		if (string [i] == '\\') {
+			if (!string [++i])
+				break;
+			*p++ = string [i];
+		} else
+			*p++ = string [i];
+	}
+	*p = '\0';
+
+	return escaped;
+}
+
+/**
+ * bonobo_moniker_client_new_from_name:
+ * @name: the display name of a moniker
+ * @ev: a corba exception environment 
+ * 
+ *  This routine tries to parse a Moniker in string form
+ *
+ * eg. file:/tmp/a.tar.gz#gzip:#tar:
+ *
+ * into a CORBA_Object representation of this that can
+ * later be resolved against an interface.
+ * 
+ * Return value: a new Moniker handle
+ **/
 Bonobo_Moniker
 bonobo_moniker_client_new_from_name (const CORBA_char  *name,
 				     CORBA_Environment *ev)
@@ -240,6 +442,13 @@ bonobo_moniker_client_new_from_name (const CORBA_char  *name,
 		CORBA_OBJECT_NIL, name, ev);
 }
 
+/**
+ * bonobo_moniker_client_get_name:
+ * @moniker: a moniker handle
+ * @ev: a corba exception environment 
+ * 
+ * Return value: the display name of the moniker.
+ **/
 CORBA_char *
 bonobo_moniker_client_get_name (Bonobo_Moniker     moniker,
 				CORBA_Environment *ev)
@@ -251,7 +460,7 @@ bonobo_moniker_client_get_name (Bonobo_Moniker     moniker,
 
 	name = Bonobo_Moniker_getDisplayName (moniker, ev);
 
-	if (BONOBO_EX(ev))
+	if (BONOBO_EX (ev))
 		return NULL;
 
 	return name;
@@ -264,20 +473,51 @@ init_default_resolve_options (Bonobo_ResolveOptions *options)
 	options->timeout = -1;
 }
 
+/**
+ * bonobo_moniker_client_resolve_default:
+ * @moniker: a moniker
+ * @interface_name: the name of the interface we want returned as the result 
+ * @ev: a corba exception environment 
+ * 
+ *  This resolves the moniker object against the given interface,
+ * with a default set of resolve options.
+ * 
+ * Return value: the interfaces resolved to or CORBA_OBJECT_NIL
+ **/
 Bonobo_Unknown
 bonobo_moniker_client_resolve_default (Bonobo_Moniker     moniker,
 				       const char        *interface_name,
 				       CORBA_Environment *ev)
 {
 	Bonobo_ResolveOptions options;
+	Bonobo_Unknown        retval;
+	char                 *real_if;
 	
+	g_return_val_if_fail (interface_name != NULL, CORBA_OBJECT_NIL);
 	g_return_val_if_fail (moniker != CORBA_OBJECT_NIL, CORBA_OBJECT_NIL);
 
+	real_if = get_full_interface_name (interface_name);
+
 	init_default_resolve_options (&options);
 
-	return Bonobo_Moniker_resolve (moniker, &options, interface_name, ev);
+	retval = Bonobo_Moniker_resolve (moniker, &options, real_if, ev);
+
+	g_free (real_if);
+
+	return retval;
 }
 
+/**
+ * bonobo_moniker_client_resolve_client_default:
+ * @moniker: the moniker
+ * @interface_name: the name of the interface we want returned as the result 
+ * @ev: a corba exception environment 
+ * 
+ * See: bonobo_moniker_client_resolve_default; however this version returns
+ * a BonoboObjectClient wrapped reference.
+ * 
+ * Return value: a BonoboObjectClient style reference.
+ **/
 BonoboObjectClient *
 bonobo_moniker_client_resolve_client_default (Bonobo_Moniker     moniker,
 					      const char        *interface_name,
@@ -290,7 +530,7 @@ bonobo_moniker_client_resolve_client_default (Bonobo_Moniker     moniker,
 	unknown = bonobo_moniker_client_resolve_default (
 		moniker, interface_name, ev);
 
-	if (BONOBO_EX(ev))
+	if (BONOBO_EX (ev))
 		return NULL;
 
 	if (unknown == CORBA_OBJECT_NIL)
@@ -299,6 +539,18 @@ bonobo_moniker_client_resolve_client_default (Bonobo_Moniker     moniker,
 	return bonobo_object_client_from_corba (unknown);
 }
 
+/**
+ * bonobo_get_object:
+ * @name: the display name of a moniker
+ * @interface_name: the name of the interface we want returned as the result 
+ * @ev: a corba exception environment 
+ * 
+ *  This encapsulates both the parse stage and resolve process of using
+ * a moniker, providing a simple VisualBasic like mechanism for using the
+ * object name space.
+ * 
+ * Return value: the requested interface or CORBA_OBJECT_NIL
+ **/
 Bonobo_Unknown
 bonobo_get_object (const CORBA_char *name,
 		   const char        *interface_name,
@@ -309,7 +561,7 @@ bonobo_get_object (const CORBA_char *name,
 
 	moniker = bonobo_moniker_client_new_from_name (name, ev);
 
-	if (BONOBO_EX(ev))
+	if (BONOBO_EX (ev))
 		return CORBA_OBJECT_NIL;
 
 	retval = bonobo_moniker_client_resolve_default (
@@ -317,18 +569,17 @@ bonobo_get_object (const CORBA_char *name,
 
 	bonobo_object_release_unref (moniker, ev);
 
-	if (BONOBO_EX(ev))
+	if (BONOBO_EX (ev))
 		return CORBA_OBJECT_NIL;
 	
 	return retval;
-
 }
 
 typedef struct {
 	char                *name;
 	BonoboMonikerAsyncFn cb;
 	gpointer             user_data;
-	guint                timeout_usec;
+	guint                timeout_msec;
 	Bonobo_Unknown       moniker;
 } parse_async_ctx_t;
 
@@ -414,7 +665,7 @@ async_activation_cb (CORBA_Object activated_object,
 			gpointer arg_values [2] = { &obj, &ctx->name };
 	
 			bonobo_async_invoke (&method, async_parse_cb, ctx,
-					     ctx->timeout_usec,
+					     ctx->timeout_msec,
 					     ctx->moniker, arg_values, &ev);
 			
 			if (BONOBO_EX (&ev)) {
@@ -429,10 +680,20 @@ async_activation_cb (CORBA_Object activated_object,
 	CORBA_exception_free (&ev);
 }
 
+/**
+ * bonobo_moniker_client_new_from_name_async:
+ * @name: the name
+ * @ev: a corba exception environment 
+ * @timeout_msec: the timeout in milliseconds 
+ * @cb: the async callback that gets the response
+ * @user_data: user context data to pass to that callback
+ * 
+ * An asynchronous version of new_from_name
+ **/
 void
 bonobo_moniker_client_new_from_name_async (const CORBA_char    *name,
 					   CORBA_Environment   *ev,
-					   guint                timeout_usec,
+					   guint                timeout_msec,
 					   BonoboMonikerAsyncFn cb,
 					   gpointer             user_data)
 {
@@ -455,7 +716,7 @@ bonobo_moniker_client_new_from_name_async (const CORBA_char    *name,
 	ctx->name         = g_strdup (name);
 	ctx->cb           = cb;
 	ctx->user_data    = user_data;
-	ctx->timeout_usec = timeout_usec;
+	ctx->timeout_msec = timeout_msec;
 	ctx->moniker      = CORBA_OBJECT_NIL;
 
 	if (!(iid = moniker_id_from_nickname (name))) {
@@ -497,12 +758,24 @@ resolve_async_cb (BonoboAsyncReply  *handle,
 	g_free (ctx);
 }
 
+/**
+ * bonobo_moniker_resolve_async:
+ * @moniker: the moniker to resolve
+ * @options: resolve options
+ * @interface_name: the name of the interface we want returned as the result 
+ * @ev: a corba exception environment 
+ * @timeout_msec: the timeout in milliseconds 
+ * @cb: the async callback that gets the response 
+ * @user_data: user context data to pass to that callback 
+ * 
+ * An async version of bonobo_moniker_client_resolve
+ **/
 void
 bonobo_moniker_resolve_async (Bonobo_Moniker         moniker,
 			      Bonobo_ResolveOptions *options,
 			      const char            *interface_name,
 			      CORBA_Environment     *ev,
-			      guint                  timeout_usec,
+			      guint                  timeout_msec,
 			      BonoboMonikerAsyncFn   cb,
 			      gpointer               user_data)
 {
@@ -538,14 +811,25 @@ bonobo_moniker_resolve_async (Bonobo_Moniker         moniker,
 	ctx->moniker = bonobo_object_dup_ref (moniker, ev);
 
 	bonobo_async_invoke (&method, resolve_async_cb, ctx,
-			     timeout_usec, ctx->moniker, arg_values, ev);
+			     timeout_msec, ctx->moniker, arg_values, ev);
 }
 
+/**
+ * bonobo_moniker_resolve_async_default:
+ * @moniker: 
+ * @interface_name: the name of the interface we want returned as the result 
+ * @ev: a corba exception environment 
+ * @timeout_msec: the timeout in milliseconds 
+ * @cb: the async callback that gets the response 
+ * @user_data: user context data to pass to that callback 
+ * 
+ * An async version of bonobo_moniker_client_resolve_default
+ **/
 void
 bonobo_moniker_resolve_async_default (Bonobo_Moniker       moniker,
 				      const char          *interface_name,
 				      CORBA_Environment   *ev,
-				      guint                timeout_usec,
+				      guint                timeout_msec,
 				      BonoboMonikerAsyncFn cb,
 				      gpointer             user_data)
 {
@@ -559,12 +843,12 @@ bonobo_moniker_resolve_async_default (Bonobo_Moniker       moniker,
 	init_default_resolve_options (&options);
 
 	bonobo_moniker_resolve_async (moniker, &options, interface_name,
-				      ev, timeout_usec, cb, user_data);
+				      ev, timeout_msec, cb, user_data);
 }
 
 
 typedef struct {
-	guint                timeout_usec;
+	guint                timeout_msec;
 	char                *interface_name;
 	BonoboMonikerAsyncFn cb;
 	gpointer             user_data;
@@ -604,7 +888,7 @@ get_async1_cb (Bonobo_Unknown     object,
 	} else {
                 bonobo_moniker_resolve_async_default (
 			object, ctx->interface_name, ev,
-			ctx->timeout_usec, get_async2_cb, ctx);
+			ctx->timeout_msec, get_async2_cb, ctx);
 
 		if (BONOBO_EX (ev)) {
 			ctx->cb (CORBA_OBJECT_NIL, ev, ctx->user_data);
@@ -613,11 +897,22 @@ get_async1_cb (Bonobo_Unknown     object,
 	}
 }	
 
+/**
+ * bonobo_get_object_async:
+ * @name: 
+ * @interface_name: the name of the interface we want returned as the result 
+ * @ev: a corba exception environment 
+ * @timeout_msec: the timeout in milliseconds 
+ * @cb: the async callback that gets the response 
+ * @user_data: user context data to pass to that callback 
+ * 
+ * An async version of bonobo_get_object
+ **/
 void
 bonobo_get_object_async (const CORBA_char    *name,
 			 const char          *interface_name,
 			 CORBA_Environment   *ev,
-			 guint                timeout_usec,
+			 guint                timeout_msec,
 			 BonoboMonikerAsyncFn cb,
 			 gpointer             user_data)
 {
@@ -631,9 +926,9 @@ bonobo_get_object_async (const CORBA_char    *name,
 	ctx = g_new0 (get_object_async_ctx_t, 1);
 	ctx->cb = cb;
 	ctx->user_data = user_data;
-	ctx->interface_name = g_strdup (interface_name);
-	ctx->timeout_usec = timeout_usec;
+	ctx->interface_name = get_full_interface_name (interface_name);
+	ctx->timeout_msec = timeout_msec;
 
 	bonobo_moniker_client_new_from_name_async (
-		name, ev, timeout_usec, get_async1_cb, ctx);
+		name, ev, timeout_msec, get_async1_cb, ctx);
 }
