@@ -55,6 +55,10 @@ GtkType                  bonobo_moniker_get_type            (void);
 POA_Bonobo_Moniker__epv *bonobo_moniker_get_epv             (void);
 Bonobo_Moniker           bonobo_moniker_corba_object_create (BonoboObject      *object);
 
+BonoboMoniker           *bonobo_moniker_construct           (BonoboMoniker *monike,
+							     Bonobo_Moniker corba_moniker,
+							     const char    *prefix);
+
 Bonobo_Moniker           bonobo_moniker_get_parent          (BonoboMoniker     *moniker,
 							     CORBA_Environment *ev);
 void                     bonobo_moniker_set_parent          (BonoboMoniker     *moniker,
@@ -62,12 +66,16 @@ void                     bonobo_moniker_set_parent          (BonoboMoniker     *
 							     CORBA_Environment *ev);
 
 /*
- * Common case convenience functions
+ *   Common case convenience functions, for people not
+ * overriding the base BonoboMoniker methods.
  */
-const char              *bonobo_moniker_get_name            (BonoboMoniker     *moniker,
-							     int                char_offset);
+const char              *bonobo_moniker_get_name            (BonoboMoniker     *moniker);
+
+const char              *bonobo_moniker_get_name_full       (BonoboMoniker     *moniker);
+char                    *bonobo_moniker_get_name_escaped    (BonoboMoniker     *moniker);
+
 void                     bonobo_moniker_set_name            (BonoboMoniker     *moniker,
-							     const char        *name,
+							     const char        *unescaped_name,
 							     int                num_chars);
 
 END_GNOME_DECLS
