@@ -8,6 +8,7 @@
  * Copyright 2000 Helix Code, Inc.
  */
 #include <config.h>
+#include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-print.h>
 
 #include <libgnomeprint/gnome-print.h>
@@ -163,7 +164,7 @@ bonobo_print_corba_object_create (BonoboObject *object)
 	CORBA_exception_init (&ev);
 
 	POA_Bonobo_Print__init ((PortableServer_Servant) servant, &ev);
-	if (ev._major != CORBA_NO_EXCEPTION){
+	if (BONOBO_EX (&ev)){
                 g_free (servant);
 		CORBA_exception_free (&ev);
                 return CORBA_OBJECT_NIL;
