@@ -10,22 +10,26 @@
 #include <config.h>
 #include <gtk/gtksignal.h>
 
+#include <bonobo/bonobo-context.h>
 #include <bonobo/bonobo-exception.h>
+#include <bonobo/bonobo-event-source.h>
 #include <bonobo/bonobo-moniker-util.h>
 #include <bonobo/bonobo-running-context.h>
 
 POA_Bonobo_RunningContext__vepv bonobo_running_context_vepv;
 
 typedef struct {
+	gboolean    emitted_last_unref;
 	GHashTable *objects;
 	GHashTable *keys;
 } BonoboRunningInfo;
 
 BonoboRunningInfo *bonobo_running_info = NULL;
 BonoboObject      *bonobo_running_context = NULL;
+BonoboEventSource *bonobo_running_event_source = NULL;
 
 enum {
-	LAST_REF,
+	LAST_UNREF,
 	LAST_SIGNAL
 };
 
@@ -56,12 +60,25 @@ running_info_destroy (void)
 		g_free (ri);
 	}
 	bonobo_running_info = NULL;
+
+	if (bonobo_running_context)
+		bonobo_object_unref (BONOBO_OBJECT (bonobo_running_context));
+	bonobo_running_context = NULL;
+	bonobo_running_event_source = NULL;
+}
+
+static void
+check_destroy (BonoboObject *object,
+	       gpointer      dummy)
+{
+	bonobo_running_context = NULL;
+	bonobo_running_event_source = NULL;
 }
 
 static BonoboRunningInfo *
-get_running_info (void)
+get_running_info (gboolean create)
 {
-	if (!bonobo_running_info) {
+	if (!bonobo_running_info && create) {
 		bonobo_running_info = g_new (BonoboRunningInfo, 1);
 		bonobo_running_info->objects = g_hash_table_new (NULL, NULL);
 		bonobo_running_info->keys    = g_hash_table_new (g_str_hash, g_str_equal);
@@ -75,21 +92,32 @@ get_running_info (void)
 static void
 check_empty (void)
 {
-	BonoboRunningInfo *ri = get_running_info ();
+	BonoboRunningInfo *ri = get_running_info (FALSE);
 
-	if (!bonobo_running_context)
+	if (!ri || !bonobo_running_context)
 		return;
 
-/*	if ((g_hash_table_size (ri->objects) == 0) &&
-	    (g_hash_table_size (ri->keys) == 0))
+	if (!ri->emitted_last_unref &&
+	    (g_hash_table_size (ri->objects) == 0) &&
+	    (g_hash_table_size (ri->keys) == 0)) {
+
+		ri->emitted_last_unref = TRUE;
+
 		gtk_signal_emit (GTK_OBJECT (bonobo_running_context),
-		signals [LAST_REF], NULL);*/
+				 signals [LAST_UNREF]);
+
+		g_return_if_fail (bonobo_running_event_source != NULL);
+
+		bonobo_event_source_notify_listeners (
+			bonobo_running_event_source,
+			"bonobo:last_unref", NULL, NULL);
+	}
 }
 
 void
 bonobo_running_context_add_object (CORBA_Object object)
 {
-	BonoboRunningInfo *ri = get_running_info ();
+	BonoboRunningInfo *ri = get_running_info (TRUE);
 
 	g_hash_table_insert (ri->objects, object, object);
 }
@@ -97,19 +125,22 @@ bonobo_running_context_add_object (CORBA_Object object)
 void
 bonobo_running_context_remove_object (CORBA_Object object)
 {
-	BonoboRunningInfo *ri = get_running_info ();
+	BonoboRunningInfo *ri = get_running_info (FALSE);
 
-	g_hash_table_remove (ri->objects, object);
+	if (ri) {
+		g_hash_table_remove (ri->objects, object);
 
-	check_empty ();
+		check_empty ();
+	}
 }
 
 void
 bonobo_running_context_ignore_object (CORBA_Object object)
 {
-	BonoboRunningInfo *ri = get_running_info ();
+	BonoboRunningInfo *ri = get_running_info (FALSE);
 
-	g_hash_table_remove (ri->objects, object);
+	if (ri)
+		g_hash_table_remove (ri->objects, object);
 }
 
 static void
@@ -134,7 +165,7 @@ impl_Bonobo_RunningContext_addKey (PortableServer_Servant servant,
 				   CORBA_Environment     *ev)
 {
 	char              *key_copy, *old_key;
-	BonoboRunningInfo *ri = get_running_info ();
+	BonoboRunningInfo *ri = get_running_info (TRUE);
 
 	old_key = g_hash_table_lookup (ri->keys, key);
 	if (old_key) {
@@ -151,9 +182,12 @@ impl_Bonobo_RunningContext_removeKey (PortableServer_Servant servant,
 				      const CORBA_char      *key,
 				      CORBA_Environment     *ev)
 {
-	BonoboRunningInfo *ri = get_running_info ();
+	BonoboRunningInfo *ri = get_running_info (FALSE);
 	char              *old_key;
 
+	if (!ri)
+		return;
+
 	old_key = g_hash_table_lookup (ri->keys, key);
 	if (old_key)
 		g_free (old_key);
@@ -197,9 +231,11 @@ bonobo_running_context_class_init (BonoboObjectClass *klass)
 
 	init_running_context_corba_class ();
 
-	signals [LAST_REF] = gtk_signal_new (
-		"last_ref", GTK_RUN_FIRST, object_class->type,
-		GTK_SIGNAL_OFFSET (BonoboRunningContextClass, last_ref),
+	((BonoboRunningContextClass *)klass)->last_unref = NULL;
+
+	signals [LAST_UNREF] = gtk_signal_new (
+		"last_unref", GTK_RUN_FIRST, object_class->type,
+		GTK_SIGNAL_OFFSET (BonoboRunningContextClass, last_unref),
 		gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
 
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
@@ -214,7 +250,7 @@ bonobo_running_context_get_type (void)
                 GtkTypeInfo info = {
                         "BonoboRunningContext",
                         sizeof (BonoboRunningContext),
-                        sizeof (BonoboObjectClass),
+                        sizeof (BonoboRunningContextClass),
                         (GtkClassInitFunc) bonobo_running_context_class_init,
                         (GtkObjectInitFunc) NULL,
                         NULL, /* reserved 1 */
@@ -275,5 +311,23 @@ bonobo_running_context_new (void)
         bonobo_running_context =
 		bonobo_object_construct (object, corba_running_context);
 
+	bonobo_running_event_source = bonobo_event_source_new ();
+	bonobo_running_context_ignore_object (
+		bonobo_object_corba_objref (BONOBO_OBJECT (
+			bonobo_running_event_source)));
+	bonobo_event_source_ignore_listeners (bonobo_running_event_source);
+
+	bonobo_object_add_interface (BONOBO_OBJECT (bonobo_running_context),
+				     BONOBO_OBJECT (bonobo_running_event_source));
+
+	gtk_signal_connect (GTK_OBJECT (bonobo_running_context), "destroy",
+			    (GtkSignalFunc) check_destroy, NULL);
+
 	return bonobo_running_context;
 }
+
+BonoboObject *
+bonobo_context_running_get (void)
+{
+	return bonobo_running_context_new ();
+}
