@@ -64,7 +64,7 @@ bonobo_transient_servant_locator_preinvoke (PortableServer_Servant servant_manag
 	 * Grab the Property name and the Property Bag.
 	 */
 	object_name = PortableServer_ObjectId_to_string (oid, ev);
-	if (BONOBO_EX(ev)) {
+	if (BONOBO_EX (ev)) {
 		CORBA_free (object_name);
 		g_warning ("BonoboPropertyBag: Could not get property name from Object ID");
 		return NULL;
@@ -169,7 +169,23 @@ bonobo_transient_get_servant_locator_vepv (void)
 	return vepv;
 }
 
-static BonoboTransient *
+/**
+ * bonobo_transient_construct:
+ * @transient: the BonoboTransient to construct
+ * @parent_poa: the POA where the object is created, CORBA_OBJECT_NIL for the default Bonobo POA.
+ * @new_servant: A function pointer used to incarnate servants on demand.
+ * @destroy_servant: A function pointer used to destroy the on-demand server.
+ * @data: data passed to the @new_servant and @destroy_servant functions.
+ *
+ * This function is only for wrappers and object derivation.  For normal
+ * use, please see #bonobo_transient_new.
+ *
+ * This function will return %NULL on failiure, however it is your
+ * responsibility to destroy the failed object in that case.
+ *
+ * Returns: a #BonoboTransient object (the @transient)
+ */
+BonoboTransient *
 bonobo_transient_construct (BonoboTransient          *transient,
 			    PortableServer_POA        poa,
 			    BonoboTransientServantNew new_servant,
@@ -347,7 +363,6 @@ bonobo_transient_construct (BonoboTransient          *transient,
 	success = TRUE;
 
  out:
-#ifdef ORBIT_IS_FINALLY_RELEASED_AND_I_DO_NOT_HAVE_TO_GO_THROUGH_THIS_BULLSHIT
 	if (policies->_buffer[0] != NULL) {
 		CORBA_Policy_destroy (policies->_buffer[0], &ev);
 
@@ -380,7 +395,7 @@ bonobo_transient_construct (BonoboTransient          *transient,
 			success = FALSE;
 		}
 	}
-#endif
+
 	g_free (policies->_buffer);
 	g_free (policies);
 
