@@ -18,6 +18,7 @@ extern "C" {
 
 #include <bonobo/bonobo-object.h>
 #include <bonobo/bonobo-moniker.h>
+#include <bonobo/bonobo-moniker-simple.h>
 #include <bonobo/bonobo-context.h>
 #include <bonobo/bonobo-exception.h>
 
@@ -28,7 +29,6 @@ extern "C" {
 
 #include <bonobo/bonobo-property-bag.h>
 #include <bonobo/bonobo-property-bag-xml.h>
-#include <bonobo/bonobo-property-listener.h>
 #include <bonobo/bonobo-property-bag-client.h>
 #include <bonobo/bonobo-property-control.h>
 
