@@ -12,6 +12,9 @@
 
 #include <bonobo.h>
 
+#undef BONOBO_TEXT_PLAIN_TEST_UI_HANDLER
+
+#ifdef BONOBO_TEXT_PLAIN_TEST_UI_HANDLER
 /* XPM */
 static char * bolt_xpm[] = {
 "10 18 4 1",
@@ -37,6 +40,7 @@ static char * bolt_xpm[] = {
 ".++@.     ",
 ".+@.      ",
 "...       "};
+#endif
 
 /*
  * Number of running objects
@@ -85,7 +89,7 @@ static void
 destroy_control (bonobo_object_data_t *bonobo_object_data)
 {
 	g_free (bonobo_object_data);
-} /* destroy_control */
+}
 
 static void
 free_text (bonobo_object_data_t *bonobo_object_data)
@@ -94,7 +98,7 @@ free_text (bonobo_object_data_t *bonobo_object_data)
 		 g_free (bonobo_object_data->text);
 	 bonobo_object_data->text = NULL;
 	 bonobo_object_data->text_len = 0;
-} /* free_text */
+}
 
 static void
 blank_control (bonobo_object_data_t *bonobo_object_data)
@@ -132,7 +136,7 @@ blank_control (bonobo_object_data_t *bonobo_object_data)
 #endif /* ! NO_PROGRESS_METER */
 
 	bonobo_object_data->text_widget_text_len = 0;
-} /* blank_control */
+}
 
 static void
 update_control (bonobo_object_data_t *bonobo_object_data)
@@ -164,7 +168,7 @@ update_control (bonobo_object_data_t *bonobo_object_data)
 	gtk_text_set_point (GTK_TEXT (bonobo_object_data->text_widget), 0);
 
 	gtk_text_thaw (GTK_TEXT (bonobo_object_data->text_widget));
-} /* update_control */
+}
 		  
 static void
 progressive_update (bonobo_object_data_t *bonobo_object_data, char *buff, size_t count)
@@ -216,7 +220,7 @@ progressive_update (bonobo_object_data_t *bonobo_object_data, char *buff, size_t
 
 	gtk_text_thaw (GTK_TEXT (bonobo_object_data->text_widget));
 
-} /* progressive_update */
+}
 
 /*
  * Callbacks attached to signals.
@@ -285,7 +289,7 @@ text_inserted_cb (GtkText *text_widget,
 	memcpy (bonobo_object_data->text + *position, text, length);
 
 	return FALSE;
-} /* text_inserted_cb */
+}
 
 static gboolean
 text_deleted_cb (GtkText *text,
@@ -306,8 +310,7 @@ text_deleted_cb (GtkText *text,
 						       bonobo_object_data->text_len);
 			
 	return FALSE;
-} /* text_deleted_cb */
-
+}
 
 static void
 verb_ClearText_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname)
@@ -316,20 +319,17 @@ verb_ClearText_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname
 
 	free_text (bonobo_object_data);
 	blank_control (bonobo_object_data);
-} /* verb_ClearText_cb */
-
-/*
- * GNOME::BonoboObject
- *
- * These functions implement the GNOME::BonoboObject interface.
- */
+}
 
 static void
 create_control_menus (bonobo_object_data_t *bonobo_object_data)
 {
 	BonoboControl *control = bonobo_object_data->bonobo_object;
 	Bonobo_UIContainer remote_uic;
+
+#ifdef BONOBO_TEXT_PLAIN_TEST_UI_HANDLER
 	BonoboUINode *parent, *node;
+#endif
 
 	/*
 	 * Get our container's UIContainer server.
@@ -356,8 +356,6 @@ create_control_menus (bonobo_object_data_t *bonobo_object_data)
 	 */
 	bonobo_object_release_unref (remote_uic, NULL);
 
-#undef BONOBO_TEXT_PLAIN_TEST_UI_HANDLER
-
 #ifdef BONOBO_TEXT_PLAIN_TEST_UI_HANDLER
 	parent = bonobo_ui_util_new_menu (TRUE, "text-plain", "_text-plain",
 					  "text-plain component", NULL);
