@@ -43,8 +43,8 @@ verb_HelpAbout_cb (BonoboUIComponent *component, Frame *frame, const char *cname
 		  "Copyright (C) 1999 Maurer IT Systemlsungen"
 		  "                     ",
 		  authors,
-		  _("This program is part of the GNOME project for "
-		    "Linux. Gnome Shell comes with ABSOLUTELY NO "
+		  _("This program is part of the GNOME project. "
+		    "Gnome Shell comes with ABSOLUTELY NO "
 		    "WARRANTY. This is free software, and you are "
 		    "welcome to redistribute it under the conditions "
 		    "of the GNU General Public License. "
