@@ -343,12 +343,15 @@ file_open_cb (GtkWidget *widget, Frame *frame)
 {
 	gchar *name;
 
-	name = strdup(gtk_file_selection_get_filename(app.fs));
+	name = g_strdup (gtk_file_selection_get_filename (app.fs));
 	gtk_widget_destroy (GTK_WIDGET(app.fs));
 	app.fs = NULL;
-	if (!name) return;
 
-	file_open(frame, name);
+	if (!name)
+		return;
+
+	file_open (frame, name);
+
 	g_free(name);
 }
 
@@ -381,7 +384,7 @@ verb_LaunchControl_cb (BonoboUIComponent *component, Frame *frame, const char *c
 	BufferSelect_CbData *cb_data;
 	GtkWidget *selector;
 	Buffer *buffer;
-	gchar *goad_id, *name;
+	gchar *oaf_iid, *name;
 	gint pos = -1, n;
 
 	g_return_if_fail(component != NULL);
@@ -394,16 +397,16 @@ verb_LaunchControl_cb (BonoboUIComponent *component, Frame *frame, const char *c
 		return;
 
 	if (n != 0) {
-	    gtk_widget_destroy (selector);
-	    return;
+		gtk_widget_destroy (selector);
+		return;
 	}
 
-	goad_id = bonobo_selector_get_selected_id (BONOBO_SELECTOR (selector));
+	oaf_iid = bonobo_selector_get_selected_id (BONOBO_SELECTOR (selector));
 	name = bonobo_selector_get_selected_name (BONOBO_SELECTOR (selector));
 
 	gtk_widget_destroy (selector);
 
-	if (!(buffer = buffer_create_for_control (goad_id))) {
+	if (!(buffer = buffer_create_for_control (oaf_iid))) {
 		g_warning ("unable to create the bonobo component");
 		return;
 	}
@@ -416,16 +419,15 @@ verb_LaunchControl_cb (BonoboUIComponent *component, Frame *frame, const char *c
 	cb_data->frame = frame;
 	cb_data->buffer = buffer;
 
-	g_free (goad_id);
+	g_free (oaf_iid);
 	g_free (name);
 
 	bonobo_ui_component_add_verb_full (frame->component, buffer->verb,
 					   (BonoboUIVerbFn) verb_BufferSelect_cb,
 					   cb_data, (GDestroyNotify) g_free);
 
-	if ((view_frame = get_active_view_frame (frame))) {
+	if ((view_frame = get_active_view_frame (frame)))
 		pos = view_remove (frame, view_frame);
-	}
 
 	buffer_add_view (buffer, frame, pos);
 	update_buffer_menu ();
@@ -439,7 +441,7 @@ verb_LaunchEmbeddable_cb (BonoboUIComponent *component, Frame *frame, const char
 	BufferSelect_CbData *cb_data;
 	GtkWidget *selector;
 	Buffer *buffer;
-	gchar *goad_id, *name;
+	gchar *oaf_iid, *name;
 	gint pos = -1, n;
 
 	g_return_if_fail(component != NULL);
@@ -456,12 +458,12 @@ verb_LaunchEmbeddable_cb (BonoboUIComponent *component, Frame *frame, const char
 	    return;
 	}
 
-	goad_id = bonobo_selector_get_selected_id (BONOBO_SELECTOR (selector));
+	oaf_iid = bonobo_selector_get_selected_id (BONOBO_SELECTOR (selector));
 	name = bonobo_selector_get_selected_name (BONOBO_SELECTOR (selector));
 
 	gtk_widget_destroy (selector);
 
-	if (!(buffer = buffer_create (goad_id))) {
+	if (!(buffer = buffer_create (oaf_iid))) {
 		g_warning ("unable to create the bonobo component");
 		return;
 	}
@@ -474,16 +476,15 @@ verb_LaunchEmbeddable_cb (BonoboUIComponent *component, Frame *frame, const char
 	cb_data->frame = frame;
 	cb_data->buffer = buffer;
 
-	g_free (goad_id);
+	g_free (oaf_iid);
 	g_free (name);
 
 	bonobo_ui_component_add_verb_full (frame->component, buffer->verb,
 					   (BonoboUIVerbFn) verb_BufferSelect_cb,
 					   cb_data, (GDestroyNotify) g_free);
 
-	if ((view_frame = get_active_view_frame (frame))) {
+	if ((view_frame = get_active_view_frame (frame)))
 		pos = view_remove (frame, view_frame);
-	}
 
 	buffer_add_view (buffer, frame, pos);
 	update_buffer_menu ();
